@@ -1,6 +1,23 @@
 #ifndef __FWEELIN_AUDIOIO_H
 #define __FWEELIN_AUDIOIO_H
 
+/* Copyright 2004-2011 Jan Pekau
+   
+   This file is part of Freewheeling.
+   
+   Freewheeling is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
+   
+   Freewheeling is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
+   You should have received a copy of the GNU General Public License
+   along with Freewheeling.  If not, see <http://www.gnu.org/licenses/>. */
+
 extern "C"
 {
 #include <jack/jack.h>
@@ -16,7 +33,7 @@ class Processor;
 
 class AudioIO {
 public:
-  AudioIO(Fweelin *app) : sync_start_frame(0), timebase_master(0), sync_active(0), app(app) {};
+  AudioIO(Fweelin *app) : sync_start_frame(0), timebase_master(0), sync_active(0), audio_thread(0), app(app) {};
 
   // Open up system level audio
   int open ();
@@ -37,8 +54,8 @@ public:
 
   // Timebase (jack transport sync) callback
   static void timebase_callback(jack_transport_state_t state, 
-				jack_nframes_t nframes, 
-				jack_position_t *pos, int new_pos, void *arg);
+                                jack_nframes_t nframes, 
+                                jack_position_t *pos, int new_pos, void *arg);
     
   // Sampling rate change callback
   static int srate_callback (nframes_t nframes, void *arg);
@@ -101,6 +118,8 @@ public:
   char sync_active;     // Nonzero if sync is active
   char transport_roll;  // Nonzero if the transport is rolling
 
+  volatile pthread_t audio_thread;  // RT audio thread (created by JACK)
+
   // Pointer to the main app
   Fweelin *app;
   // Processor which is basically the root of the signal flow
