@@ -1,6 +1,23 @@
 #ifndef __FWEELIN_BLOCK_H
 #define __FWEELIN_BLOCK_H
 
+/* Copyright 2004-2011 Jan Pekau
+   
+   This file is part of Freewheeling.
+   
+   Freewheeling is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
+   
+   Freewheeling is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
+   You should have received a copy of the GNU General Public License
+   along with Freewheeling.  If not, see <http://www.gnu.org/licenses/>. */
+
 #include <vorbis/codec.h>
 #include <vorbis/vorbisfile.h>
 #include <vorbis/vorbisenc.h>
@@ -14,15 +31,18 @@
 #include "fweelin_audioio.h"
 #include "fweelin_mem.h"
 #include "fweelin_event.h"
+#include "elastin/elastin.h"
 
 // Types of audio codecs supported
-typedef enum {UNKNOWN = -1,
-              FIRST_FORMAT = 0, 
-	      VORBIS = 0,
-	      WAV = 1,
-	      FLAC = 2,
-	      AU = 3,
-	      END_OF_FORMATS = 4} codec;
+typedef enum {
+  UNKNOWN = -1,
+  FIRST_FORMAT = 0, 
+  VORBIS = 0,
+  WAV = 1,
+  FLAC = 2,
+  AU = 3,
+  END_OF_FORMATS = 4
+} codec;
 
 #ifndef MIN
 #define MIN(a,b) ((a) < (b) ? (a) : (b))
@@ -87,7 +107,19 @@ class TimeMarker : public Preallocated {
  public:
   TimeMarker(nframes_t markofs = 0, long data = 0) : markofs(markofs), 
     data(data), next(0) {};
+  // FWMEM_DEFINE_DELBLOCK;
+  
+  virtual void Recycle() {
+    markofs = 0;
+    data = 0;
+    next = 0;
+  };
 
+  // Block mode allocate
+  /* virtual Preallocated *NewInstance() { 
+    return ::new TimeMarker[GetMgr()->GetBlockSize()]; 
+    }; */
+  
   virtual Preallocated *NewInstance() { return ::new TimeMarker(); };
 
   nframes_t markofs; // Marker position measured in samples
@@ -149,7 +181,7 @@ public:
   // either smooth the beginning into the end (smoothtype == 1)
   // or smooth the end into the beginning     (smoothtype == 0)
   void Smooth(char smoothtype = 1, 
-	      nframes_t smoothlen = AUDIOBLOCK_SMOOTH_ENDPOINTS_LEN);
+              nframes_t smoothlen = AUDIOBLOCK_SMOOTH_ENDPOINTS_LEN);
 
   // Erases this whole audioblock chain from first to last
   // Also erases any extended data!
@@ -168,7 +200,7 @@ public:
   // Finds the audioblock and offset into that block that correspond
   // to the provided absolute offset into this chain 
   void SetPtrsFromAbsOffset(AudioBlock **ptr, nframes_t *blkofs, 
-			    nframes_t absofs);
+                            nframes_t absofs);
 
   // Generates a subchain of AudioBlocks by copying the samples between offsets
   // from & to.. offsets expressed absolutely with reference to
@@ -178,7 +210,7 @@ public:
   // the block and the beginning up to toofs (wrap case)
   // Realtime safe?
   AudioBlock *GenerateSubChain(nframes_t fromofs, nframes_t toofs,
-			       char copystereo);
+                               char copystereo);
 
   // Removes the last 'hacklen' samples from this block chain
   // Returns nonzero on error!
@@ -219,14 +251,25 @@ class BED_ExtraChannel : public Preallocated, public BlockExtendedData {
     *origbuf;    // Original unmodified sample pointer
 };
 
-// Iterates through a list of audio blocks, given the first,
-// and produces & stores n-sized fragments on demand
-class AudioBlockIterator {
+// Iterator for storing/extracting data in audio blocks.
+// Freewheeling stores audio in small blocks, which are linked together
+// in a chain. 
+//
+// The main functions for iteration are GetFragment (retrieves an audio
+// fragment from block(s)), PutFragment (stores an audio fragment into
+// block(s)), and NextFragment (advances the iterator).
+//
+// The iterator can also perform rate scaling.
+//
+// When rate scaling, the iterator writes to a different block chain than
+// it reads from. The chain can then grow/shrink as the rate scaling 
+// changes.
+class AudioBlockIterator /*: public Elastin_SampleFeed*/ {
 public:
   // Optionally pass preallocatedtype for extrachannel if you want
   // extra channels to be added as needed when putting fragments
   AudioBlockIterator(AudioBlock *firstblock, nframes_t fragmentsize,
-		     PreallocatedType *pre_extrachannel = 0);
+                     PreallocatedType *pre_extrachannel = 0);
   ~AudioBlockIterator();
 
   inline float round(float num) {
@@ -251,15 +294,21 @@ public:
   // Jumps to an absolute offset within the blockchain
   void Jump(nframes_t ofs);
 
-  // PutFragment puts the specified fragment back into this AudioBlock
+  // PutFragment stores the specified fragment back into this AudioBlock
   // (optional right channel)
   // Returns nonzero if the end of the block is reached, and we wrap to next  
   // Optional size_override puts a different size fragment into the block-
   // any size can be put so long as the chain is long enough
   // Optional wait_alloc waits for allocation of new extra channels if needed
   // (don't use this flag in RT!)
+  
+  // If rate scaling is active, the iterator always writes to a new
+  // block, because the length of the newly stored sample may be
+  // different than the length of the original (see 'write block' 
+  // variables). No scaling is done in PutFragment, just a choice of
+  // which block chain to write to
   int PutFragment (sample_t *frag_l, sample_t *frag_r, 
-		   nframes_t size_override = 0, char wait_alloc = 0);
+                   nframes_t size_override = 0, char wait_alloc = 0);
 
   // Returns the current fragment of audio
   // Points frag_l and frag_r buffers to the current fragment
@@ -272,24 +321,42 @@ public:
   // becomes the new end of the chain
   void EndChain();
 
-  inline nframes_t GetTotalLength2Cur() { return curcnt; }
+  inline nframes_t GetTotalLength2Cur() { return (nframes_t) curcnt; }
   inline char IsStopped() { return stopped; };
   inline void Stop() { stopped = 1; };
 
   // Returns the block currently being iterated through
   inline AudioBlock *GetCurBlock() { return curblock; };
 
-  // Optional right channel for the block we are iterating
+ private:
+
+  // Optional right audio channel for the block we are iterating
   PreallocatedType *pre_extrachannel; // Preallocator for 2nd channel blocks
   BED_ExtraChannel *currightblock,
-    *nextrightblock;
+    *nextrightblock,
+
+    // Values for write block
+    *currightblock_w,
+    *nextrightblock_w;
 
   AudioBlock *curblock,
-    *nextblock;
-  nframes_t curblkofs,
+    *nextblock,
+
+    // Values for write block
+    *curblock_w,
+    *nextblock_w;
+
+  double curblkofs, // Use doubles for position (rate scale needs them)
     nextblkofs,
     curcnt,
-    nextcnt;
+    nextcnt,
+
+    // Values for write block
+    curblkofs_w,
+    nextblkofs_w,
+    curcnt_w,
+    nextcnt_w;
+
 
   // Buffers for storing smaller fragments from within AudioBlocks
   nframes_t fragmentsize;
@@ -306,8 +373,13 @@ enum ManagedChainType {
   T_MC_BlockRead,
   T_MC_BlockWrite,
   T_MC_HiPri,
-  T_MC_StripeBlock,
-  T_MC_DelayProcessorCall
+  T_MC_StripeBlock
+};
+
+// Status of managed chain
+enum ManagedChainStatus {
+  T_MC_Running,
+  T_MC_PendingDelete
 };
 
 // Generic class specifying a chain of blocks & iterator to manage
@@ -347,6 +419,7 @@ public:
   AudioBlock *b;
   AudioBlockIterator *i;
   
+  ManagedChainStatus status;
   ManagedChain *next;
 };
 
@@ -456,12 +529,12 @@ class VorbisEncoder : public iFileEncoder {
 
   // Vorbis encoder stuff
   ogg_stream_state os; /* take physical pages, weld into a logical
-			  stream of packets */
+                          stream of packets */
   ogg_page         og; /* one Ogg bitstream page.  Vorbis packets are inside */
   ogg_packet       op; /* one raw packet of data for decode */
   
   vorbis_info      vi; /* struct that stores all the static vorbis bitstream
-			  settings */
+                          settings */
   vorbis_comment   vc; /* struct that stores all the user comments */
 
   vorbis_dsp_state vd; /* central working state for the packet->PCM decoder */
@@ -559,7 +632,7 @@ class AutoWriteControl {
   // decide which loops to save, while the BlockManager thread does the work
   // in the background
   virtual void GetWriteBlock(FILE **out, AudioBlock **b, 
-			     AudioBlockIterator **i, nframes_t *len) = 0;
+                             AudioBlockIterator **i, nframes_t *len) = 0;
 };
 
 class AutoReadControl {
@@ -593,7 +666,7 @@ class BlockReadManager : public ManagedChain {
   const static int NUM_DECODE_PASSES = 100;
 
   BlockReadManager(FILE *in = 0, AutoReadControl *arc = 0, 
-		   BlockManager *bmg = 0, nframes_t peaksavgs_chunksize = 0);
+                   BlockManager *bmg = 0, nframes_t peaksavgs_chunksize = 0);
   virtual ~BlockReadManager();
 
   virtual Preallocated *NewInstance() { return ::new BlockReadManager(); };
@@ -655,8 +728,8 @@ class BlockWriteManager : public ManagedChain {
     ENCODE_CROSSOVER_LEN = 1000;
 
   BlockWriteManager(FILE *out = 0, AutoWriteControl *awc = 0, 
-		     BlockManager *bmg = 0, AudioBlock *b = 0, 
-		     AudioBlockIterator *i = 0);
+                     BlockManager *bmg = 0, AudioBlock *b = 0, 
+                     AudioBlockIterator *i = 0);
   virtual ~BlockWriteManager();
 
   virtual Preallocated *NewInstance() { return ::new BlockWriteManager(); };
@@ -675,11 +748,11 @@ class BlockWriteManager : public ManagedChain {
       printf("DISK: Blocks deleted while saving- abort!\n");
       End();
       if (awc != 0)
-	// Auto save, so keep running
-	return 0;
+        // Auto save, so keep running
+        return 0;
       else 
-	// Single save, so stop
-	return 1;
+        // Single save, so stop
+        return 1;
     }
     else
       return 0;
@@ -687,7 +760,7 @@ class BlockWriteManager : public ManagedChain {
 
   // Start encoding the given block chain to the given file
   void Start(FILE *new_out = 0, AudioBlock *new_b = 0, 
-	     AudioBlockIterator *new_i = 0, nframes_t new_len = 0);
+             AudioBlockIterator *new_i = 0, nframes_t new_len = 0);
 
   // Ends writing
   void End();
@@ -713,8 +786,8 @@ class BlockWriteManager : public ManagedChain {
 class PeaksAvgsManager : public ManagedChain {
  public:
   PeaksAvgsManager(BlockManager *bmg = 0, 
-		   AudioBlock *b = 0, AudioBlockIterator *i = 0, 
-		   char grow = 0) : 
+                   AudioBlock *b = 0, AudioBlockIterator *i = 0, 
+                   char grow = 0) : 
     ManagedChain(b,i), bmg(bmg), runmax(0), runmin(0), runtally(0), 
     lastcnt(0), chunkcnt(0), stereo(0), grow(grow), go(1), ended(0) {};
   virtual ~PeaksAvgsManager();
@@ -784,7 +857,7 @@ class PeaksAvgsManager : public ManagedChain {
 class HiPriManagedChain : public ManagedChain {
  public:
   HiPriManagedChain(void *trigger = 0, 
-		    AudioBlock *b = 0, AudioBlockIterator *i = 0) :
+                    AudioBlock *b = 0, AudioBlockIterator *i = 0) :
     ManagedChain(b,i), trigger(trigger), lastcnt(0) {};
 
   virtual Preallocated *NewInstance() { 
@@ -815,8 +888,8 @@ class HiPriManagedChain : public ManagedChain {
 class StripeBlockManager : public HiPriManagedChain {
  public:
   StripeBlockManager(PreallocatedType *pre_tm = 0,
-		     void *trigger = 0, 
-		     AudioBlock *b = 0, AudioBlockIterator *i = 0) :
+                     void *trigger = 0, 
+                     AudioBlock *b = 0, AudioBlockIterator *i = 0) :
     HiPriManagedChain(trigger,b,i), pre_tm(pre_tm) {};
 
   // Call before starting!
@@ -838,38 +911,6 @@ class StripeBlockManager : public HiPriManagedChain {
   PreallocatedType *pre_tm;
 };
 
-// This manager calls processor method p::ProcessorCall when 
-// BlockManager is called with the specified trigger pointer
-// ProcessorCall returns the new data value to be passed to it
-// next time- or -1 to stop this manager!
-class DelayProcessorCallManager : public HiPriManagedChain {
- public:
-  DelayProcessorCallManager(void *trigger = 0, Processor *p = 0,
-			    int data = 0) :
-    HiPriManagedChain(trigger,0,0), p(p), data(data) {};
-
-  virtual Preallocated *NewInstance() { 
-    return ::new DelayProcessorCallManager();
-  };
-
-  virtual ManagedChainType GetType() { return T_MC_DelayProcessorCall; };
-
-  virtual int Manage();
-
-  virtual int RefDeleted(void *ref) { 
-    if (ref == b || ref == i || ref == p) 
-      // Block, iterator, or processor gone! End this manager!
-      return 1;
-    else
-      return 0;
-  };
-
-  // Processor to call
-  Processor *p;
-  // Data to pass it
-  int data;
-};
-
 // BlockManager handles different maintenance tasks related to
 // audio blocks. 
 //
@@ -878,9 +919,6 @@ class DelayProcessorCallManager : public HiPriManagedChain {
 //
 // It also handles -time-critical events-, such as firing off pulse sync
 // messages. This second function may soon be moved to the EventManager.
-//
-// TODO: Doublecheck for possible threading issues.
-//       accessing lists from nonRT and RT threads simultaneously?
 class BlockManager {
 public:
   BlockManager (Fweelin *app);
@@ -899,23 +937,16 @@ public:
   // for the specified Block & Iterator
   // We compute as the currenty iterated position advances
   PeaksAvgsManager *PeakAvgOn (AudioBlock *b, AudioBlockIterator *i, 
-			       char grow = 0);
+                               char grow = 0);
   void PeakAvgOff (AudioBlock *b);
 
   // Stripes the specified chain with TimeMarkers according to the specified
   // trigger. Works in conjunction with RT threads that call HiPriTrigger.
   void StripeBlockOn (void *trigger, AudioBlock *b, 
-		      AudioBlockIterator *i);
+                      AudioBlockIterator *i);
   // Removes striping from the specified trigger on blockchain b
   void StripeBlockOff (void *trigger, AudioBlock *b);
 
-  // Removes all delayprocessorcallmanagers on the given trigger
-  // that call the given processor
-  void FlushDelayProcessorCall (void *trigger, Processor *p);
-
-  void AddDelayProcessorCall (void *trigger, Processor *p, int data);
-  void DelDelayProcessorCall (DelayProcessorCallManager *m);
-
   // Notify all Managers that the object pointed to has been deleted-
   // To avoid broken dependencies
   void RefDeleted (void *ref);
@@ -937,21 +968,23 @@ public:
 
   void DelManager (ManagedChain **first, ManagedChain *m);
   void AddManager (ManagedChain **first, ManagedChain *nw);
+  void AddHiManager (HiPriManagedChain **first, HiPriManagedChain *nw);
   void RefDeleted (ManagedChain **first, void *ref);
+  void HiRefDeleted (HiPriManagedChain **first, void *ref);
 
   // Delete a managed chain for block o and manager type t
   // If t is T_MC_None, removes the first managed chain for 'o' 
   // of any type
   void DelManager (ManagedChain **first, AudioBlock *o,
-		   ManagedChainType t = T_MC_None);
+                   ManagedChainType t = T_MC_None);
 
   // Delete a hiprimanaged chain for block o and manager type t
   // with specified trigger.
   // If t is T_MC_None, removes the first managed chain for 'o' 
   // with specified trigger, of any type
   void DelHiManager (HiPriManagedChain **first, 
-		     AudioBlock *o,
-		     ManagedChainType t, void *trigger);
+                     AudioBlock *o,
+                     ManagedChainType t, void *trigger);
 
   // NEED TO MAKE/USE GENERALIZED LIST CLASS
   // ^^ speed issues?
@@ -962,14 +995,14 @@ public:
   HiPriManagedChain *himanageblocks;
 
   pthread_t manage_thread;
+  pthread_mutex_t manage_thread_lock;
   int threadgo;
 
   // ****************** PREALLOCATED TYPE MANAGERS
   PreallocatedType *pre_growchain,
     *pre_peaksavgs,
     *pre_hipri,
-    *pre_stripeblock,
-    *pre_delayprocessorcall;
+    *pre_stripeblock;
 
   // Parent app
   Fweelin *app;
