@@ -1,6 +1,23 @@
 #ifndef __FWEELIN_BROWSER_H
 #define __FWEELIN_BROWSER_H
 
+/* Copyright 2004-2011 Jan Pekau
+   
+   This file is part of Freewheeling.
+   
+   Freewheeling is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
+   
+   Freewheeling is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
+   You should have received a copy of the GNU General Public License
+   along with Freewheeling.  If not, see <http://www.gnu.org/licenses/>. */
+
 #include "fweelin_event.h"
 #include "fweelin_config.h"
 
@@ -100,6 +117,8 @@ class ItemRenamer : public EventHook {
 
   ItemRenamer(Fweelin *app, RenameCallback *cb, char *oldname);
 
+  virtual ~ItemRenamer() {};
+
   // Hook events for typing new names
   virtual char HookEvent(Event *ev, EventProducer *from);
 
@@ -110,7 +129,7 @@ class ItemRenamer : public EventHook {
       rui.rename_cursor_blinktime = t;
       rui.rename_cursor_toggle = (rui.rename_cursor_toggle ? 0 : 1);
     }
-	  
+          
     return &rui;
   };
   inline char IsRenaming() { return renaming; };
@@ -142,10 +161,12 @@ class ItemRenamer : public EventHook {
 };
 
 class Browser : public FloDisplay, public EventListener, public EventProducer,
-		public RenameCallback {
+                public RenameCallback {
  public:
-  Browser(BrowserItemType btype, char xpand, int xpand_x1, int xpand_y1,
-	  int xpand_x2, int xpand_y2, float xpand_delay) : 
+  Browser (int iid, 
+           BrowserItemType btype, char xpand, int xpand_x1, int xpand_y1,
+           int xpand_x2, int xpand_y2, float xpand_delay) : 
+    FloDisplay(iid),
     renamer(0), app(0), btype(btype), callback(0), first(0), cur(0),
     xpand_x1(xpand_x1), xpand_y1(xpand_y1), 
     xpand_x2(xpand_x2), xpand_y2(xpand_y2), xpand_centery(-1), 
@@ -189,7 +210,7 @@ class Browser : public FloDisplay, public EventListener, public EventProducer,
   // Write the name to outbuf, with max maxlen characters.
   // Returns nonzero if we used a 'default' name.
   char GetDisplayName(char *filename, time_t *filetime,
-		      char *outbuf, int maxlen);
+                      char *outbuf, int maxlen);
 
   ItemRenamer *renamer; // Renamer instance, or null if we are not renaming
   virtual void ItemRenamed(char *nw); // Callback for renamer
@@ -198,7 +219,7 @@ class Browser : public FloDisplay, public EventListener, public EventProducer,
   // of an item. For example, when a Loop in memory is renamed, we also
   // rename it on disk, and the loop browser must be notified of this new name.
   void ItemRenamedOnDisk(char *old_filename, char *new_filename, 
-			 char *new_name);
+                         char *new_name);
 
   inline BrowserItemType GetType() { return btype; };
   virtual FloDisplayType GetFloDisplayType() { return FD_Browser; };
@@ -308,9 +329,10 @@ class Browser : public FloDisplay, public EventListener, public EventProducer,
 
 class LoopTray : public Browser {
  public:
-  LoopTray(BrowserItemType btype, char xpand, int xpand_x1, int xpand_y1,
-	   int xpand_x2, int xpand_y2, int loopsize) : 
-    Browser(btype,xpand,xpand_x1,xpand_y1,xpand_x2,xpand_y2,0.0),
+  LoopTray (int iid,
+            BrowserItemType btype, char xpand, int xpand_x1, int xpand_y1,
+            int xpand_x2, int xpand_y2, int loopsize) : 
+    Browser(iid,btype,xpand,xpand_x1,xpand_y1,xpand_x2,xpand_y2,0.0),
     loopsize(loopsize), basepos(0), iconsize(0), 
     resize_win(RS_Off), touchtray(0), loopmap(0) {};
   virtual ~LoopTray();
@@ -354,8 +376,9 @@ class LoopTray : public Browser {
 class CombiZone {
  public:
   void SetupZone (int kr_lo = 0, int kr_hi = 0,
-		  char port_r = 0, int port = 0,
-		  int bank = -1, int prog = -1, int channel = 0) {
+                  char port_r = 0, int port = 0,
+                  int bank = -1, int prog = -1, int channel = 0,
+                  char bypasscc = 0, int bypasschannel = -1, float bypasstime1 = 0.0, float bypasstime2 = 10.0) {
     this->kr_lo = kr_lo;
     this->kr_hi = kr_hi;
     this->port_r = port_r;
@@ -363,6 +386,11 @@ class CombiZone {
     this->bank = bank;
     this->prog = prog;
     this->channel = channel;
+    
+    this->bypasscc = bypasscc;
+    this->bypasschannel = bypasschannel;
+    this->bypasstime1 = bypasstime1;
+    this->bypasstime2 = bypasstime2;
   };
 
   int kr_lo,   // Bottom key for this zone
@@ -375,6 +403,11 @@ class CombiZone {
     bank, // Bank select for zone (-1 for none)
     prog, // Program change for zone (-1 for none)
     channel; // Transmit channel # for zone
+    
+  char bypasscc;      // MIDI CC to send for bypass
+  int bypasschannel;  // MIDI channel to send bypass to (or -1 to use regular channel)
+  float bypasstime1,  // Length of time (s) before auto-bypass when notes are sustaining (with or without pedal)
+    bypasstime2;      // Length of time (s) before auto-bypass when notes have been released
 };
 
 // Small class to encapsulate patches
@@ -382,8 +415,9 @@ class PatchItem : public BrowserItem {
 public:
 
   PatchItem (int id = 0, int bank = 0, int prog = 0, int channel = 0,
-	     char *name = 0) :
+             char *name = 0, char bypasscc = 0, int bypasschannel = -1, float bypasstime1 = 0.0, float bypasstime2 = 10.0) :
     BrowserItem(name), id(id), bank(bank), prog(prog), channel(channel),
+    bypasscc(bypasscc), bypasschannel(bypasschannel), bypasstime1(bypasstime1), bypasstime2(bypasstime2), 
     zones(0), numzones(0) {};
   virtual ~PatchItem() {
     if (zones != 0)
@@ -413,6 +447,11 @@ public:
     prog, // Program change for patch
     channel; // Channel # for patch
   
+  char bypasscc;      // MIDI CC to send for bypass
+  int bypasschannel;  // MIDI channel to send bypass to (or -1 to use regular channel)
+  float bypasstime1,  // Length of time (s) before auto-bypass when notes are sustaining (with or without pedal)
+    bypasstime2;      // Length of time (s) before auto-bypass when notes have been released
+  
   // For a combination patch, we have an array of all zones
   CombiZone *zones;
   int numzones;
@@ -427,12 +466,16 @@ public:
 #define MIDI_BANKCHANGE_LSB 32
 class PatchBank {
  public:
-  PatchBank (int port, char suppresschg) : port(port), 
-    suppresschg(suppresschg),
+  PatchBank (int port, int tag, char suppresschg) : port(port), 
+    tag(tag), suppresschg(suppresschg),
     first(0), cur(0), next(0) {};
 
-  int port;           // MIDI output port for this patch bank
+  int port,           // MIDI output port for this patch bank
                       // (or -1 for internal FluidSynth patch bank)
+    tag;              // Tag for identifying patchbank- can be 
+                      // used by the user config to change FW behavior when
+                      // a given patchbank is active
+  
   char suppresschg;   // Suppress program/bank change messages being sent 
                       // for this patch bank?
 
@@ -449,10 +492,11 @@ class PatchBrowser : public Browser {
   // Every DIV_SPACING patches, we insert a divider in the browser
   const static int DIV_SPACING = 10;
   
-  PatchBrowser(BrowserItemType btype, char xpand, int xpand_x1, int xpand_y1,
-	       int xpand_x2, int xpand_y2, float xpand_delay) : 
-    Browser(btype,xpand,xpand_x1,xpand_y1,xpand_x2,xpand_y2,xpand_delay),
-    pb_first(0), pb_cur(0), num_pb(0) {};
+  PatchBrowser (int iid,
+                BrowserItemType btype, char xpand, int xpand_x1, int xpand_y1,
+                int xpand_x2, int xpand_y2, float xpand_delay) : 
+    Browser(iid,btype,xpand,xpand_x1,xpand_y1,xpand_x2,xpand_y2,xpand_delay),
+    pb_first(0), pb_cur(0), pb_cur_tag(-1), num_pb(0) {};
   virtual ~PatchBrowser();
 
   virtual void ClearAllItems();
@@ -476,7 +520,7 @@ class PatchBrowser : public Browser {
     else {
       PatchBank *pb_i = pb_first;
       while (pb_i->next != 0)
-	pb_i = pb_i->next;
+        pb_i = pb_i->next;
       pb_i->next = pb_cur = pb;
     }
     
@@ -484,7 +528,8 @@ class PatchBrowser : public Browser {
     cur = pb->cur;
     first = pb->first;
 
-    SetMIDIEcho();
+    pb_cur_tag = pb->tag;
+    SetMIDIForPatch();
 
     UnlockBrowser();
   };
@@ -508,7 +553,8 @@ class PatchBrowser : public Browser {
     cur = pb->cur;
     first = pb->first;
 
-    SetMIDIEcho();
+    pb_cur_tag = pb->tag;
+    SetMIDIForPatch();
 
     UnlockBrowser();
   };
@@ -521,19 +567,63 @@ class PatchBrowser : public Browser {
 
   inline PatchBank *GetCurPatchBank() const { return pb_cur; };
 
-  // Sets the right MIDI port(s) and channel(s) for echo based on this
+  // Sets the right MIDI port(s) and channel(s) for echo & auto-bypass based on this
   // patches' settings
-  void SetMIDIEcho();
+  void SetMIDIForPatch();
 
  protected:
 
-  virtual void ItemBrowsed() { SetMIDIEcho(); };
+  // Select patch automatically when we browse to it
+  // ** Disabled
+  // virtual void ItemBrowsed() { SetMIDIForPatch(); };
 
  private:
 
   PatchBank *pb_first, // List of patchbanks
     *pb_cur;           // Current patchbank
-  int num_pb;          // Number of patchbanks defined
+  int pb_cur_tag,      // Tag from current patchbank 
+    num_pb;            // Number of patchbanks defined
+};
+
+class FloDisplaySnapshots : public FloDisplay, public RenameCallback
+{
+ public:
+  FloDisplaySnapshots (Fweelin *app, int iid) : FloDisplay(iid), renamer(0),
+    app(app), firstidx(0), numdisp(-1) {
+    pthread_mutex_init (&snaps_lock,0);
+  };
+  ~FloDisplaySnapshots() { 
+     pthread_mutex_destroy (&snaps_lock);
+  };
+  
+  virtual FloDisplayType GetFloDisplayType() { return FD_Snapshots; };
+
+  virtual void Draw(SDL_Surface *screen);
+
+  ItemRenamer *renamer; // Renamer instance, or null if we are not renaming
+  int rename_idx;       // Index of snapshot we are renaming
+  virtual void ItemRenamed(char *nw); // Callback for renamer
+  
+  // Rename the snapshot with given index
+  void Rename (int idx);
+
+  inline void LockSnaps() {
+    pthread_mutex_lock (&snaps_lock);
+  };
+  inline void UnlockSnaps() {
+    pthread_mutex_unlock (&snaps_lock);
+  };
+
+  Fweelin *app;
+  int firstidx,          // Index of first snapshot to display
+    numdisp;             // Number of snapshots to display
+  int sx, sy,            // Size of snapshots list
+    margin;              // Margin for text
+    
+  protected:
+  
+  pthread_mutex_t snaps_lock; // A way to lock up snapshot display so two threads
+                              // don't race on it
 };
 
 #endif
