@@ -1,6 +1,23 @@
 #ifndef __FWEELIN_CONFIG_H
 #define __FWEELIN_CONFIG_H
 
+/* Copyright 2004-2011 Jan Pekau
+   
+   This file is part of Freewheeling.
+   
+   Freewheeling is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
+   
+   Freewheeling is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
+   You should have received a copy of the GNU General Public License
+   along with Freewheeling.  If not, see <http://www.gnu.org/licenses/>. */
+
 #include <SDL/SDL.h>
 
 #include <libxml/xmlmemory.h>
@@ -27,19 +44,25 @@
 #define FWEELIN_FILE_BROWSER_DIVISION_TIME 3600
 
 #define FWEELIN_CONFIG_DIR  ".fweelin"
-#define FWEELIN_CONFIG_FILE ".fweelin.rc"
+#define FWEELIN_CONFIG_FILE "fweelin.xml"
+#define FWEELIN_CONFIG_EXT  ".xml"
 #define FWEELIN_CONFIG_HELP_TOKEN "HELP:"
 
 #define FWEELIN_OUTPUT_STREAM_NAME "live"
 #define FWEELIN_OUTPUT_TIMING_EXT  ".wav.usx"
 #define FWEELIN_OUTPUT_LOOP_NAME   "loop"
 #define FWEELIN_OUTPUT_SCENE_NAME  "scene"
+#define FWEELIN_OUTPUT_SNAPSHOT_NAME "snapshot"
+#define FWEELIN_OUTPUT_LOOPSNAPSHOT_NAME "loopsnap"
 #define FWEELIN_OUTPUT_DATA_EXT    ".xml"
 
 // Console sequence for error color
 #define FWEELIN_ERROR_COLOR_ON "\033[31;1m"
 #define FWEELIN_ERROR_COLOR_OFF "\033[0m"
 
+// Interface ID assigned to first non-switchable interface
+#define NS_INTERFACE_START_ID 1000
+
 #ifdef __MACOSX__
 // On Linux, FWEELIN_DATADIR refers to /usr/local/share/fweelin as set by autoconf
 // On Mac, we store our data in the Resource directory of the fweelin.app bundle. 
@@ -49,7 +72,7 @@ extern char *FWEELIN_DATADIR;
 
 // On Linux, VERSION is defined within configure.ac
 // For Mac OS, we set it here:
-#define VERSION "0.5.5"
+#define VERSION "0.6"
 #endif
 
 #include "fweelin_datatypes.h"
@@ -62,6 +85,7 @@ class SDLIO;
 class MidiIO;
 class CircularMap;
 class PatchBrowser;
+class ParamSetBank;
 
 // ****************** CONFIG CLASSES
 
@@ -171,18 +195,13 @@ class DynamicToken {
 // controls basic user interface
 class EventBinding {
  public:
-  EventBinding(char all = 0) : 
-    boundproto(0), heldprereq(0), all(all), echo(0), 
+  EventBinding() : 
+    boundproto(0), echo(0), 
     tokenconds(0), paramsets(0), continued(0), next(0) {};
   virtual ~EventBinding();
 
   Event *boundproto; // Prototype instance of the output event
 
-  // We can specify that for the event to be triggered, a set of
-  // keyboard keys must be held down.
-  SDLKeyList *heldprereq; // These keys must be held
-  char all; // Nonzero if all keys must be held, zero if any one is enough
-
   // Nonzero if input events should be rebroadcast even
   // if they are consumed in this binding
   char echo;
@@ -212,7 +231,7 @@ class InputMatrix : public EventProducer, public EventListener {
 
   InputMatrix(Fweelin *app);
   virtual ~InputMatrix();
-  
+
   // Sets the given variable to the given value- string is interpreted
   // based on variable type
   void SetVariable (UserVariable *var, char *value);
@@ -221,7 +240,7 @@ class InputMatrix : public EventProducer, public EventListener {
   void CreateVariable (xmlNode *declare);
 
   // Called during configuration to bind input controllers to events
-  void CreateBinding (xmlNode *binding);
+  void CreateBinding (int interfaceid, xmlNode *binding);
 
   // Are the conditions in the EventBinding bind matched by the
   // given input event and user variables?
@@ -245,7 +264,7 @@ class InputMatrix : public EventProducer, public EventListener {
   // The expression may also reference parameters in event 'ref'
   // and these references will be extracted
   ParsedExpression *ParseExpression(char *str, Event *ref, 
-				    char enable_keynames = 0);
+                                    char enable_keynames = 0);
 
  private:
 
@@ -265,7 +284,7 @@ class InputMatrix : public EventProducer, public EventListener {
   // Parses the given token (no math ops!) into dst
   // Correctly identifies when variables or event parameters are referenced
   void ParseToken(char *str, CfgToken *dst, Event *ref, 
-		  char enable_keynames = 0);
+                  char enable_keynames = 0);
 
   // Stores in ptr the value val given that ptr is of type dtype
   void StoreParameter(char *ptr, CoreDataType dtype, UserVariable *val);
@@ -276,15 +295,16 @@ class InputMatrix : public EventProducer, public EventListener {
 
   // Scans in the given binding for settings for output event parameters
   // and sets us up to handle those
-  void CreateParameterSets (EventBinding *bind, xmlNode *binding, 
-			    Event *input, int contnum);
+  void CreateParameterSets (int interfaceid,
+                            EventBinding *bind, xmlNode *binding, 
+                            Event *input, int contnum);
 
   // Scans in the given binding for conditions on input event parameters 
   // or user variables, and sets us up to handle those
   // Returns the hash index for this binding, based on an indexed parameter,
   // or 0 if this binding is not indexed
-  int CreateConditions (EventBinding *bind, xmlNode *binding, 
-			Event *input, int paramidx);
+  int CreateConditions (int interfaceid, EventBinding *bind, 
+                        xmlNode *binding, Event *input, int paramidx);
 
   // Traverses through the list of event bindings beginning at 'start'
   // looking for a binding that matches current user variables and input
@@ -302,6 +322,8 @@ class FloLayoutElementGeometry {
  public:  
   FloLayoutElementGeometry() : next(0) {};
 
+  virtual ~FloLayoutElementGeometry() {};
+
   // Draw this element to the given screen-
   // implementation given in videoio.cc
   virtual void Draw(SDL_Surface *screen, SDL_Color clr) = 0;
@@ -325,7 +347,7 @@ class FloLayoutBox : public FloLayoutElementGeometry {
   // element geometry
   virtual char Inside(int x, int y) {
     if (x >= left && x <= right &&
-	y >= top && y <= bottom)
+        y >= top && y <= bottom)
       return 1;
     else
       return 0;
@@ -341,7 +363,7 @@ class FloLayoutElement {
  public:  
   FloLayoutElement() : id(0), name(0), nxpos(0), nypos(0), bx(0.0), by(0.0), 
     loopmap(0), loopx(0), loopy(0), loopsize(0), geo(0), next(0) {};
-  ~FloLayoutElement() {
+  virtual ~FloLayoutElement() {
     if (name != 0)
       delete[] name;
     // Erase geometries
@@ -359,7 +381,7 @@ class FloLayoutElement {
     FloLayoutElementGeometry *cur = geo;
     while (cur != 0) {
       if (cur->Inside(x,y))
-	return 1; // Inside this geo, so inside this element
+        return 1; // Inside this geo, so inside this element
       cur = cur->next;
     }    
 
@@ -378,7 +400,7 @@ class FloLayoutElement {
   int loopx, loopy, // Position of loop graphic for the element
     loopsize; // Size of loop graphic (diameter)
 
-  // Geo describes how to draw this interface element
+  // Geo describes how to draw this element
   FloLayoutElementGeometry *geo;
 
   // Next element 
@@ -388,7 +410,7 @@ class FloLayoutElement {
 // The user can define the onscreen layout for loops-- see config file!
 class FloLayout {
  public:
-  FloLayout() : id(0), xpos(0), ypos(0), loopids(0,0),
+  FloLayout() : id(0), iid(0), xpos(0), ypos(0), loopids(0,0),
     name(0), nxpos(0), nypos(0), elems(0), show(1), showlabel(1),
     showelabel(1), next(0) {};
   ~FloLayout() {
@@ -403,7 +425,9 @@ class FloLayout {
     }
   };
   
-  int id, // User refers to a layout by ID
+  int id, // User refers to a layout by layout ID
+    iid,  // Interface id. Interface id + layout id uniquely identify 
+          // a layout
     xpos, ypos; // Base location on screen for this layout
   Range loopids; // Range of loopids that map to interface elements
   char *name; // ex PC Keyboard, MIDI Footpedal
@@ -455,14 +479,17 @@ class FloFont {
 
 enum FloDisplayType {
   FD_Unknown,
-  FD_Browser
+  FD_Browser,
+  FD_Snapshots,
+  FD_ParamSet,
 };
 
 // List of variable displays used in video
 // There are different types of displays, this is a base class
 class FloDisplay {
  public:
-  FloDisplay() : id(-1), exp(0), font(0), title(0), xpos(0), ypos(0), show(1),
+  FloDisplay (int iid) : iid(iid), 
+    id(-1), exp(0), font(0), title(0), xpos(0), ypos(0), show(1), forceshow(0),
     next(0) {};
   virtual ~FloDisplay() {
     if (title != 0)
@@ -477,21 +504,59 @@ class FloDisplay {
 
   virtual FloDisplayType GetFloDisplayType() { return FD_Unknown; };
 
-  int id;                // Way to refer to this display from events
+  // Set whether this display is showing
+  virtual void SetShow(char show) { this->show = show; };
+
+  int iid,  // Interface id. Interface id + display id uniquely identify 
+            // a display
+    id;     // Display ID
 
   ParsedExpression *exp; // Expression which evaluates to a value to display
   FloFont *font;         // Font for text
   char *title;           // Title to be displayed
   int xpos, ypos;        // Onscreen location for display
-  char show;             // Show (nonzero) or hide (zero) display
-
+  char show,             // Show (nonzero) or hide (zero) display
+    forceshow;           // Force display to show?
+    
   FloDisplay *next;
 };
 
+// Panel is a 'container' that holds several child displays and surrounds it with a frame
+// Allows easy showing and hiding of a number of displays together
+class FloDisplayPanel : public FloDisplay
+{
+ public:
+  FloDisplayPanel (int iid) : FloDisplay(iid), sx(100), sy(100), num_children(0), child_displays(0) {};
+  ~FloDisplayPanel () {
+    if (child_displays != 0)
+      delete[] child_displays;
+  };
+
+  virtual void Draw(SDL_Surface *screen);
+
+  // Set whether this display is showing
+  virtual void SetShow(char show) {
+    this->show = show;
+
+    // Show/hide child displays
+    for (int i = 0; i < num_children; i++)
+      child_displays[i]->SetShow(show);
+  };
+
+  int sx, sy; // Size of panel
+  int margin; // Margin for displays inside panel
+
+  // Array of pointers to all child displays. Child displays are still stored in the master list of
+  // displays, and drawn independently of the panel.
+  int num_children;
+  FloDisplay **child_displays;
+};
+
 // Text display shows the value of expression 'exp' as onscreen text
 class FloDisplayText : public FloDisplay 
 {
  public:
+  FloDisplayText (int iid) : FloDisplay(iid) {};
 
   virtual void Draw(SDL_Surface *screen);
 };
@@ -501,6 +566,7 @@ class FloDisplayText : public FloDisplay
 class FloDisplaySwitch : public FloDisplay 
 {
  public:
+  FloDisplaySwitch (int iid) : FloDisplay(iid) {};
 
   virtual void Draw(SDL_Surface *screen);
 };
@@ -510,7 +576,7 @@ class FloDisplaySwitch : public FloDisplay
 class FloDisplayCircleSwitch : public FloDisplay 
 {
  public:
-  FloDisplayCircleSwitch() : rad1(0), rad0(0), flash(0), prevnonz(0), 
+  FloDisplayCircleSwitch (int iid) : FloDisplay(iid), rad1(0), rad0(0), flash(0), prevnonz(0), 
     nonztime(0.) {};
 
   virtual void Draw(SDL_Surface *screen);
@@ -528,7 +594,7 @@ class FloDisplayCircleSwitch : public FloDisplay
 class FloDisplayTextSwitch : public FloDisplay 
 {
  public:
-  FloDisplayTextSwitch() : text1(0), text0(0) {};
+  FloDisplayTextSwitch (int iid) : FloDisplay(iid), text1(0), text0(0) {};
 
   virtual void Draw(SDL_Surface *screen);
 
@@ -545,13 +611,18 @@ enum CfgOrientation {
 class FloDisplayBar : public FloDisplay 
 {
  public:
-  FloDisplayBar() : orient(O_Vertical), barscale(1.0), thickness(10) {};
+  FloDisplayBar (int iid) : FloDisplay(iid), 
+    orient(O_Vertical), barscale(1.0), thickness(10), dbscale(0), marks(0), maxdb(0) {};
 
   virtual void Draw(SDL_Surface *screen);
 
   CfgOrientation orient; // Orientation of bar
   float barscale; // Scaling factor for size of bar
   int thickness;  // Thickness of bar
+  char dbscale,   // If nonzero, this bar maps a linear amplitude variable to a logarithmic scale bar
+                  // If zero, there is a linear mapping to the bar
+    marks;        // If nonzero, calibration marks are shown (dB scale only)
+  float maxdb;    // (dbscale) Maximum dB level shown
 };
 
 // Bar-switch display shows the value of expression 'exp' as a bar and changes the color of the bar
@@ -559,8 +630,8 @@ class FloDisplayBar : public FloDisplay
 class FloDisplayBarSwitch : public FloDisplayBar
 {
  public:
-  FloDisplayBarSwitch() : switchexp(0), color(1), calibrate(0),
-    cval(0.0) {};
+  FloDisplayBarSwitch (int iid) : FloDisplayBar(iid), 
+    switchexp(0), color(1), calibrate(0), cval(0.0) {};
   virtual ~FloDisplayBarSwitch() {
     if (switchexp != 0)
       delete switchexp;
@@ -572,8 +643,21 @@ class FloDisplayBarSwitch : public FloDisplayBar
                                // to appear bright, zero values cause a dim, faded bar
   
   int color; // Color of bar-switch (index of hardcoded color)
-  char calibrate; // Nonzero shows calibration value on barswitch
-  float cval; // Calibration value
+  char calibrate; // Nonzero shows calibration value on barswitch & changes color when level exceeds calibration value
+  float cval; // Calibration value (linear)
+};
+
+class FloDisplaySquares : public FloDisplay 
+{
+ public:
+  FloDisplaySquares (int iid) : FloDisplay(iid), orient(O_Horizontal) {};
+
+  virtual void Draw(SDL_Surface *screen);
+
+  CfgOrientation orient; // Orientation of bar
+  float v1, v2,          // Value corresponding to first and last square
+    sinterval;           // 1 square for every 'sinterval' change in value
+  int sx, sy;            // Square size
 };
 
 // FluidSynth config
@@ -665,16 +749,39 @@ class FloConfig {
   // Start function, called shortly before Fweelin begins running
   void Start() { im.Start(); };
 
-  // Configure bindings between events and their triggers
-  void ConfigureEventBindings(xmlDocPtr doc, xmlNode *events);
+  // Send start-interface event to all interfaces
+  void StartInterfaces ();
 
+  // Copy config file from shared folder
+  // Optionally copy all config files
+  void CopyConfigFile (char *cfgname, char copyall);
+
+  // Prepare to load configuration file 'cfgname' 
+  // Finds the file in one of several places, and copies it to
+  // the config folder. Returns the path name if found, or null if not found
+  char *PrepareLoadConfigFile (char *cfgname, char basecfg);
+  
+  // Configure bindings between events and their triggers
+  void ConfigureEventBindings(xmlDocPtr doc, xmlNode *events, 
+                              int interfaceid = 0, char firstpass = 0);
+
+  // Configuring displays
+  void SetupParamSetBank(xmlDocPtr doc, xmlNode *banknode, ParamSetBank *bank);
+  FloDisplay *SetupParamSet(xmlDocPtr doc, xmlNode *paramset, int interfaceid);
+
+  // Configuration sections
   void ConfigureElement(xmlDocPtr doc, xmlNode *elemn, 
-			FloLayoutElement *elem, float xscale, float yscale);
+                        FloLayoutElement *elem, float xscale, float yscale);
   void ConfigureLayout(xmlDocPtr doc, xmlNode *layn, 
-		       FloLayout *lay, float xscale, float yscale);
+                       FloLayout *lay, float xscale, float yscale);
+  void ConfigureDisplay_Common(xmlNode *disp, FloDisplay *nw, FloDisplayPanel *parent);
+  void ConfigureDisplay(xmlDocPtr doc, xmlNode *disp, int interfaceid, FloDisplayPanel *parent = 0);
   void ConfigurePatchBanks(xmlNode *pb, PatchBrowser *br);
-  void ConfigureVideo(xmlDocPtr doc, xmlNode *vid);
-  void ConfigureGeneral(xmlDocPtr doc, xmlNode *gen);
+  void ConfigureGraphics(xmlDocPtr doc, xmlNode *vid, int interfaceid = 0);
+  void ConfigureBasics(xmlDocPtr doc, xmlNode *gen);
+  void ConfigureInterfaces (xmlDocPtr doc, xmlNode *ifs, char firstpass);
+  void ConfigureRoot (xmlDocPtr doc, xmlNode *root, int interfaceid = 0,
+                      char firstpass = 0);
   
   // Is node 'n' a comment with help information? If so, add to our
   // internal help list
@@ -684,6 +791,9 @@ class FloConfig {
   // can then refer to the variable
   UserVariable *AddEmptyVariable(char *name);
 
+  // Returns a pointer to the given variable
+  UserVariable *GetVariable(char *name);
+
   // Makes the given variable into a system variable by linking it to
   // the pointer
   void LinkSystemVariable(char *name, CoreDataType type, char *ptr);
@@ -749,11 +859,20 @@ class FloConfig {
   // Is input/output #n stereo?
   inline char IsStereoInput(int n) { return ms_inputs[n]; };
   inline char IsStereoOutput(int n) { return IsStereoMaster(); };
-  char *ms_inputs; // Zero or nonzero for each input- is this input stereo?
-
+  inline char IsInputMonitoring(int n) { return monitor_inputs[n]; };
+  char *ms_inputs,    // Zero or nonzero for each input- is this input stereo?
+    *monitor_inputs;  // Nonzero if monitoring is enabled for this input
   // Is FreeWheeling running in stereo or completely in mono?
   char IsStereoMaster();
 
+  // Stream settings
+  inline char IsStreamFinal() { return stream_final; };
+  inline char IsStreamLoops() { return stream_loops; };
+  inline char IsStreamInputs(int n) { return stream_inputs[n]; };
+  char stream_final,
+    stream_loops,
+    *stream_inputs;
+
   // Number of external audio inputs into FreeWheeling (specified in config file)
   // AudioIO may add its own inputs internal to FreeWheeling 
   // (for example, softsynth)
@@ -776,6 +895,10 @@ class FloConfig {
   inline float GetLimiterReleaseRate() { return limiterreleaserate; };
   float limiterreleaserate;
   
+  // Logarithmic fader settings
+  inline float GetFaderMaxDB() { return fadermaxdb; };
+  float fadermaxdb;
+    
   // File format to save loops to
   inline codec GetLoopOutFormat() { return loopoutformat; };
   codec loopoutformat;
@@ -797,7 +920,7 @@ class FloConfig {
   inline codec GetCodecFromName (const char *n) {
     for (codec i = FIRST_FORMAT; i < END_OF_FORMATS; i = (codec) (i+1))
       if (!strcasecmp(n,GetCodecName(i)))
-	return i;
+        return i;
     
     return UNKNOWN;
   };
@@ -858,11 +981,20 @@ class FloConfig {
 
   // Graphical displays
   inline FloDisplay *GetDisplays() { return displays; };
-  inline FloDisplay *GetDisplayById(int id) {
+  inline FloDisplay *GetDisplayById (int iid, int id) {
     FloDisplay *cur = displays;
     while (cur != 0) {
-      if (cur->id == id)
-	return cur;
+      if (cur->iid == iid && cur->id == id)
+        return cur;
+      cur = cur->next;
+    }
+    return 0;
+  };
+  inline FloDisplay *GetDisplayByType (FloDisplayType typ) {
+    FloDisplay *cur = displays;
+    while (cur != 0) {
+      if (cur->GetFloDisplayType() == typ)
+        return cur;
       cur = cur->next;
     }
     return 0;
@@ -903,6 +1035,8 @@ class FloConfig {
   int GetFluidChannel() { return fschannel; };
   char fsstereo;
   char GetFluidStereo() { return fsstereo; };
+  float fstuning;
+  float GetFluidTuning() { return fstuning; };
   FluidSynthParam *fsparam;
   FluidSynthParam *GetFluidParam() { return fsparam; };
   void AddFluidParam(FluidSynthParam *nw) {
@@ -911,7 +1045,7 @@ class FloConfig {
     else {
       FluidSynthParam *cur = fsparam;
       while (cur->next != 0)
-	cur = cur->next;
+        cur = cur->next;
       cur->next = nw;
     }
   };
@@ -922,7 +1056,7 @@ class FloConfig {
     else {
       FluidSynthSoundFont *cur = fsfont;
       while (cur->next != 0)
-	cur = cur->next;
+        cur = cur->next;
       cur->next = nw;
     }
   };
@@ -938,12 +1072,22 @@ class FloConfig {
 
   int status_report;
 #define FS_REPORT_BLOCKMANAGER 1
-  
+
+  // Total number of interfaces defined in config 
+  int numinterfaces, // Switchable interfaces
+                     // (range 1<=i<=numinterfaces) 
+    numnsinterfaces; // Nonswitchable interfaces
+
+  // Maximum number of snapshots user can create
+  inline int GetMaxSnapshots() { return max_snapshots; };
+  int max_snapshots;
+
   // Seconds of fixed audio history 
   const static float AUDIO_MEMORY_LEN;
   // # of audio blocks to preallocate
   const static int NUM_PREALLOCATED_AUDIO_BLOCKS;
-
+  // # of time markers to preallocate
+  const static int NUM_PREALLOCATED_TIME_MARKERS;
 };
 
 #endif
