@@ -1,6 +1,26 @@
 #ifndef __FWEELIN_CORE_DSP_H
 #define __FWEELIN_CORE_DSP_H
 
+/* Copyright 2004-2011 Jan Pekau
+   
+   This file is part of Freewheeling.
+   
+   Freewheeling is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
+   
+   Freewheeling is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
+   You should have received a copy of the GNU General Public License
+   along with Freewheeling.  If not, see <http://www.gnu.org/licenses/>. */
+
+#include <string>
+#include <sstream>
+
 #include "fweelin_core.h"
 #include "fweelin_event.h"
 #include "fweelin_block.h"
@@ -13,6 +33,13 @@
 #define MAX(a,b) ((a) > (b) ? (a) : (b))
 #endif
 
+extern int math_gcd (int a, int b);
+extern int math_lcm (int a, int b);
+
+// Macros for converting between linear amplitude and dB
+#define DB2LIN(db) (powf(10.0f, (db) * 0.05f))
+#define LIN2DB(amp) (20.0f * log10f(amp))
+
 class Fweelin;
 class Loop;
 class AudioBlock;
@@ -20,6 +47,15 @@ class AudioBlockIterator;
 class PeaksAvgsManager;
 class TimeMarker;
 class InputSettings;
+class AudioLevel;
+
+// Class for converting between dB and vertical fader levels 
+class AudioLevel {
+  public:
+
+  static float fader_to_dB (float level, float maxDb);
+  static float dB_to_fader (float dB, float maxDb);
+};
 
 // Audio buffers encapsulates all buffers going into & out of
 // a processor-- allowing for multiple ins and outs to be passed
@@ -45,7 +81,7 @@ class AudioBuffers {
 
   // Mixes the selected inputs to dest (array of 2 channels)
   void MixInputs (nframes_t len, sample_t **dest, InputSettings *iset,
-		  float inputvol, char compute_stats);
+                  float inputvol, char compute_stats);
 
   // Get number of internal audio inputs into FreeWheeling
   static inline int GetIntAudioIns() { 
@@ -61,11 +97,11 @@ class AudioBuffers {
 
   Fweelin *app; // Main app
 
-  int numins, // Total number of ins
-    numins_ext, // Number of external inputs
-    numouts;  // & outs
-  sample_t **ins[2], // 2 list of input sample buffers (mono/left and right) 
-    **outs[2]; // & 2 lists of output sample buffers (mono/left and right)
+  int numins,        // Total number of inputs (including internal Fluidsynth)
+    numins_ext,      // Number of external inputs (not including internal Fluidsynth)
+    numouts;         // & outs
+  sample_t **ins[2], // 2 lists of input sample buffers (mono/left and right)
+    **outs[2];       // & 2 lists of output sample buffers (mono/left and right)
 };
 
 // Settings for each input coming into FreeWheeling
@@ -113,7 +149,7 @@ class InputSettings {
   };
 
   // (de)Select input
-  void SelectInput(int n, char selected) { 
+  inline void SelectInput(int n, char selected) {
     if (n >= 0 || n < numins) {
       selins[n] = selected;
     } else {
@@ -121,7 +157,7 @@ class InputSettings {
     }
   };
   // Is input n selected?
-  char InputSelected(int n) { 
+  inline char InputSelected(int n) {
     if (n >= 0 || n < numins) {
       return selins[n];
     } else {
@@ -135,15 +171,8 @@ class InputSettings {
 
   // Set input volume for input n
   void AdjustInputVol(int n, float adjust); 
-  void SetInputVol(int n, float vol) { 
-    if (n >= 0 || n < numins) {
-      invols[n] = vol; 
-      dinvols[n] = 1.0;
-    } else {
-      printf("CORE: InputSettings- input number %d not in range.\n",n);
-    }
-  };
-  float GetInputVol(int n) { 
+  void SetInputVol(int n, float vol, float logvol);
+  inline float GetInputVol(int n) {
     if (n >= 0 || n < numins) {
       return invols[n];
     } else {
@@ -151,7 +180,9 @@ class InputSettings {
       return 0;
     }
   };
-  float *GetInputVols() { return invols; };
+  inline float *GetInputVols() { return invols; };
+
+  inline int GetNumInputs() { return numins; };
 
   // Copy contents of settings from source object- don't fuss with pointers
   inline void operator = (InputSettings &src) {
@@ -192,6 +223,8 @@ enum SyncStateType {
 };
 
 // Handles realtime DSP from a signal chain
+// *** Consider making this an RT allocated type--
+// because right now, triggering loops actually allocates memory
 class Processor : public EventProducer {
 public:
   const static float MIN_VOL;
@@ -208,14 +241,6 @@ public:
 
   Fweelin *getAPP() { return app; };
 
-  // This method provides communication between hi priority block
-  // managers and processors- when triggered,
-  // (not type specific, so that pointer is left void)
-  // Through BlockManager, DelayProcessorCallManager then calls
-  // this method in real-time
-  // Used for syncronizing to later events such as downbeats
-  virtual int ProcessorCall(void *trigger, int data) { return -1; };
-
   // This is called by RootProcessor when a processor is flagged-
   // pending delete and should no longer perform any processing
   virtual void Halt() {};
@@ -244,29 +269,48 @@ public:
 class ProcessorItem {
 public:
   // Processor is running, or ready to be deleted
-  const static int STATUS_GO = 0,
-    STATUS_PENDING_DELETE = 1;
+  const static int STATUS_GO = 0,   // Running
+    STATUS_LIVE_PENDING_DELETE = 1, // Call RT thread, then delete
+    STATUS_PENDING_DELETE = 2;      // Delete at next non-RT opportunity
 
   // Processor type- 
-  // Global: processor will process
-  //   after output volume transformation, and so, is not affected by it
-  // Hipriority: these processors will process
-  //   before all other processors
-  // Final: processor will process after all other processors--
-  // after TYPE_GLOBAL, after autolimit-- 
-  // it will be fed the end signal chain of all other processors *as input*
+  //
+  // Global processors get their input from the external audio inputs.
+  //   They are executed near the end of the signal chain (after output volume transformation)
+  //
+  // Hipriority: these processors will process before all other processors
+  //
+  // Final processors will process after all other processors--
+  // after TYPE_GLOBAL
+  // They will be fed the end signal chain of all other processors as input
   const static int TYPE_DEFAULT = 0,
     TYPE_GLOBAL = 1,
-    TYPE_HIPRIORITY = 2,
-    TYPE_FINAL = 3;
+    TYPE_GLOBAL_SECOND_CHAIN = 2,
+    TYPE_HIPRIORITY = 3,
+    TYPE_FINAL = 4;
 
-  ProcessorItem(Processor *p, int type = TYPE_DEFAULT) : p(p), next(0), 
-    status(STATUS_GO), type(type) {};
+  ProcessorItem(Processor *p, int type = TYPE_DEFAULT, char silent = 0) : p(p), next(0),
+    status(STATUS_GO), type(type), silent(silent) {};
 
   Processor *p;
   ProcessorItem *next;
   int status,
     type;
+  char silent;    // Nonzero if this processor should always be silent (no output)
+};
+
+class PulseSyncCallback {
+public:
+
+  virtual void PulseSync (int syncidx, nframes_t actualpos) = 0;
+};
+
+class PulseSync {
+public:
+  PulseSync (PulseSyncCallback *cb = 0, nframes_t syncpos = 0) : cb(cb), syncpos(syncpos) {};
+  
+  PulseSyncCallback *cb;  // Callback instance
+  nframes_t syncpos;      // Position where to call back
 };
 
 // Pulse defines a heartbeat for a piece
@@ -276,10 +320,12 @@ class Pulse : public Processor {
 public:
   // Length of metronome strike sound in samples
   const static nframes_t METRONOME_HIT_LEN;
+  // Length of metronome tone sound in samples
+  const static nframes_t METRONOME_TONE_LEN;
   // Initial metronome volume
   const static float METRONOME_INIT_VOL;
-  // Maximum number of sync positions
-  const static int MAX_SYNC_POS = 100;
+  // Maximum number of user-defined pulse sync callbacks
+  const static int MAX_SYNC_POS = 1000;
 
   Pulse(Fweelin *app, nframes_t len, nframes_t startpos);
   ~Pulse();
@@ -304,48 +350,48 @@ public:
 
   // These methods add and remove sync positions.
   // A pulse sends out an RT PulseSync event whenever any of these
-  // positions is reached.
-  inline int AddSyncPos(nframes_t pos) { // Returns sync index of new position
+  // positions is reached. (RT Safe, but can only be called from one thread- RT audio thread)
+  inline int AddPulseSync(PulseSyncCallback *cb, nframes_t pos) { // Returns sync index of new position
     // Check position
     if (pos >= len) {
       printf("PULSE: Sync position adjusted for really short loop "
-	     "(pos %d, len %d)\n",pos,len);
+             "(pos %d, len %d)\n",pos,len);
       pos = len-1;
     }
 
     // First, search for unfilled positions in our array
     int i = 0;
-    while (i < numsyncpos && syncpos[i] != 0)
+    while (i < numsyncpos && syncpos[i].cb != 0)
       i++;
     if (i < numsyncpos) {
       // Position found, use this index
-      syncpos[i] = pos;
+      syncpos[i] = PulseSync(cb,pos);
       return i;
     } else {
       // No holes found, add to end of array
       if (numsyncpos >= MAX_SYNC_POS) {
-	printf("PULSE: Too many sync positions.\n");
-	return -1;
+        printf("PULSE: Too many sync positions.\n");
+        return -1;
       } else {
-	int ret = numsyncpos;
-	syncpos[numsyncpos++] = pos;
-	return ret;
+        int ret = numsyncpos;
+        syncpos[numsyncpos++] = PulseSync(cb,pos);
+        return ret;
       }
     }
   };
  
-  // Removes sync position at index syncidx
-  inline void DelSyncPos(int syncidx) {
+  // Removes sync position at index syncidx (RT Safe, but can only be called from one thread- RT audio thread)
+  inline void DelPulseSync (int syncidx) {
     if (syncidx < 0 || syncidx >= numsyncpos)
       printf("PULSE: Invalid sync position %d (0->%d).\n",syncidx,numsyncpos);
     else {
       if (syncidx+1 == numsyncpos) {
-	// Position exists on the end of array- shrink
-	syncpos[syncidx] = 0;
-	numsyncpos--;
+        // Position exists on the end of array- shrink
+        syncpos[syncidx] = PulseSync();
+        numsyncpos--;
       } else
-	// Position exists in the middle of the array- create a hole
-	syncpos[syncidx] = 0;
+        // Position exists in the middle of the array- create a hole
+        syncpos[syncidx] = PulseSync();
     }
   };
 
@@ -380,8 +426,19 @@ public:
   // Returns length of pulse in frames
   inline nframes_t GetLength() { return len; };
 
+  inline int GetLongCount_Len() { return lc_len; };  
+  inline int GetLongCount_Cur() { return lc_cur; };
+  inline float GetLongCount_CurPct() { return (float) lc_cur + GetPct(); };
+
+  int ExtendLongCount (long nbeats, char endjustify);
+  void ResetLongCount() { lc_len = 1; };
+  
   nframes_t len, // Length of one revolution of this pulse in samples
-    curpos; // Current position in samples into this pulse
+    curpos;      // Current position in samples into this pulse
+  
+  int lc_len,    // Length of long count
+    lc_cur;      // Current position in long count
+    
   char wrapped,  // Wrapped?
     stopped;     // Stopped?
 
@@ -396,35 +453,70 @@ public:
   nframes_t prevtap; // samplecnt @ previous tap
 
   // Metronome
-  sample_t *metro; // Sample data for metronome strike
-  nframes_t metroofs, // Current position into metronome sample
-    metrolen;         // Length of metronome sample
+  sample_t *metro,    // Sample data for metronome strike
+    *metrohitone,     // Metronome hi tone
+    *metrolotone;     // Metronome lo tone
+  nframes_t metroofs, // Current position into metronome strike sample
+    metrohiofs,       // Current position into high metronome tone
+    metroloofs,       // Current position into low metronome tone
+    metrolen,         // Length of metronome strike sample
+    metrotonelen;     // Length of metronome hi/lo tone samples
   char metroactive;   // Nonzero if metronome sound is active
   float metrovol;     // Volume of metronome
 
-  nframes_t syncpos[MAX_SYNC_POS]; // Sync positions
+  PulseSync syncpos[MAX_SYNC_POS]; // Sync positions
   int numsyncpos; // Current number of sync positions
   
   SyncStateType clockrun;  // Status of MIDI clock
 };
 
+// This class implements a quick auto limiter.
+// This design does not add latency to the output. It is not a brickwall or look-ahead design.
+// It cannot guarantee that no samples are clipped.
+// It is essentially a compressor with fast attack time.
+//
+// It is intended to prevent hard overdriving of the signal chain and potential damage to audio components.
+// It does produce some distortion when peaking. It is fairly musical and low on CPU usage.
+class AutoLimitProcessor : public Processor {
+  friend class Fweelin;
+
+public:
+
+  AutoLimitProcessor(Fweelin *app);
+  virtual ~AutoLimitProcessor();
+
+  float GetLimiterVolume() { return curlimitvol; };
+  char GetLimiterFreeze() { return limiterfreeze; };
+  void SetLimiterFreeze(char set) { limiterfreeze = set; };
+  void ResetLimiter();
+
+  // This process function processes in place on the output. It does not read the input at all.
+  virtual void process(char pre, nframes_t len, AudioBuffers *ab);
+
+private:
+
+  const static float LIMITER_ATTACK_LENGTH,
+    LIMITER_START_AMP;
+  const static nframes_t LIMITER_ADJUST_PERIOD;
+  float dlimitvol,    // Limiter volume delta- how fast vol is being changed
+    limitvol,         // Target volume- amplitude needed to stop clipping
+    curlimitvol,      // Current volume- volume is moving towards limitvol
+    maxvol;           // Maximum volume found in audio coming into limiter
+  char limiterfreeze; // Nonzero if limiter is frozen- no changes in amp
+};
+
 // This is the base of signal processing tree- it connects to system level audio
 // and calls child processors which do signal processing
 // Child processes execute in parallel and their signals are summed
-class RootProcessor : public Processor {
+class RootProcessor : public Processor, public EventListener {
+#define RP_QUEUE_SIZE 200 // Number of events that can be queued up here. Things like 'starting loops playing'.
+
+  friend class Fweelin;
+
 public:
   RootProcessor(Fweelin *app, InputSettings *iset);
   virtual ~RootProcessor();
 
-  // Adds a child processor.. the processor begins processing immediately
-  // Possibly realtime safe?
-  void AddChild (Processor *o, int type = ProcessorItem::TYPE_DEFAULT);
-
-  // Removes a child processor from receiving processing time..
-  // also, deletes the child processor
-  // Realtime safe!
-  void DelChild (Processor *o);
-
   void AdjustOutputVolume(float adjust);
   void SetOutputVolume(float set) { 
     // Preprocess audio for smoothing
@@ -444,11 +536,7 @@ public:
     dinputvol = 1.0;
   };
   inline float GetInputVolume() { return inputvol; }
-
-  float GetLimiterVolume() { return curlimitvol; };
-  char GetLimiterFreeze() { return limiterfreeze; };
-  void SetLimiterFreeze(char set) { limiterfreeze = set; };
-  void ResetLimiter();
+  inline float *GetInputVolumePtr() { return &inputvol; }
 
   // Sample accurate timing is provided through samplecnt
   inline nframes_t GetSampleCnt() { return samplecnt; };
@@ -457,15 +545,34 @@ public:
   // passing abchild audio buffers to the processors and optionally mixing
   // into the main output buffers ab 
   void processchain(char pre, nframes_t len, AudioBuffers *ab,
-		    AudioBuffers *abchild, const int ptype, 
-		    const char mixintoout);
+                    AudioBuffers *abchild, const int ptype, 
+                    const char mixintoout);
 
   virtual void process(char pre, nframes_t len, AudioBuffers *ab);
 
-  static void *run_cleanup_thread (void *ptr);
-      
-  pthread_t cleanup_thread;
-  int threadgo;
+  // Adds a child processor.. the processor begins processing immediately
+  // Possibly realtime safe?
+  void AddChild (Processor *o, int type = ProcessorItem::TYPE_DEFAULT, char silent = 0);
+
+  // Removes a child processor from receiving processing time..
+  // also, deletes the child processor
+  // Realtime safe!
+  void DelChild (Processor *o);
+
+  // Create ring buffers once all threads are present
+  void FinalPrep ();
+
+  void ReceiveEvent(Event *ev, EventProducer *from);
+
+private:
+
+  // Update the list of processors
+  void UpdateProcessors();
+
+  // Event queue (read in RT). Used to handle events in the RT audio thread, that are generated from multiple
+  // writers in other threads.
+  SRMWRingBuffer<Event *> *eq;
+  volatile char protect_plist;  // Nonzero if the processor list is being read and SHOULD NOT be modified
 
   // Volumes- we are responsible for adjusting volumes in RT
   InputSettings *iset;
@@ -481,20 +588,10 @@ public:
   sample_t *buf[2], *prebuf[2];
  
   // Count samples processed from start of execution
-  nframes_t samplecnt;
-
-  // Autolimiter
-  const static float LIMITER_ATTACK_LENGTH,
-    LIMITER_START_AMP; 
-  const static nframes_t LIMITER_ADJUST_PERIOD;
-  float dlimitvol, // Limiter volume delta- how fast vol is being changed
-    limitvol,      // Target volume- amplitude needed to stop clipping
-    curlimitvol,   // Current volume- volume is moving towards limitvol
-    maxvol;        // Maximum volume found in audio coming into limiter
-  char limiterfreeze; // Nonzero if limiter is frozen- no changes in amp
+  volatile nframes_t samplecnt;
 };
 
-class RecordProcessor : public Processor, public EventListener {
+class RecordProcessor : public Processor, public PulseSyncCallback {
 public:
   // Extra tail on record end facilitates smooth crossfade for
   // sync-recorded loops
@@ -513,27 +610,27 @@ public:
 
   // Recording into preexisting fixed size block
   RecordProcessor(Fweelin *app,
-		  InputSettings *iset, 
-		  float *inputvol,
-		  AudioBlock *dest, int suggest_stereo = -1);
+                  InputSettings *iset, 
+                  float *inputvol,
+                  AudioBlock *dest, int suggest_stereo = -1);
 
   // Overdubbing version of record into existing loop
   RecordProcessor(Fweelin *app,
-		  InputSettings *iset, 
-		  float *inputvol,
-		  Loop *od_loop,
-		  float od_playvol,
-		  nframes_t od_startofs,
-		  float *od_feedback);	// Pointer to value for feedback (can be continuously varied)
+                  InputSettings *iset, 
+                  float *inputvol,
+                  Loop *od_loop,
+                  float od_playvol,
+                  nframes_t od_startofs,
+                  float *od_feedback);  // Pointer to value for feedback (can be continuously varied)
 
-  // Recording new blocks, growing size as necessary	       
+  // Recording new blocks, growing size as necessary           
   RecordProcessor(Fweelin *app,
-		  InputSettings *iset, 
-		  float *inputvol,
-		  Pulse *sync = 0, 
-		  AudioBlock *audiomem = 0,
-		  AudioBlockIterator *audiomemi = 0,
-		  nframes_t peaksavgs_chunksize = 0);
+                  InputSettings *iset, 
+                  float *inputvol,
+                  Pulse *sync = 0, 
+                  AudioBlock *audiomem = 0,
+                  AudioBlockIterator *audiomemi = 0,
+                  nframes_t peaksavgs_chunksize = 0);
 
   // Destructor- executed nonRT
   ~RecordProcessor();
@@ -541,10 +638,9 @@ public:
   virtual void process(char pre, nframes_t len, AudioBuffers *ab);
 
   // In Halt() method we ensure that no stray Pulse_Syncs will be responded to
-  virtual void Halt() { sync_state = SS_ENDED; };
+  virtual void Halt() { stopped = 1; sync_state = SS_ENDED; };
 
-  virtual void ReceiveEvent(Event *ev, EventProducer *from);
-  // virtual int ProcessorCall(void *trigger, int data);
+  virtual void PulseSync (int syncidx, nframes_t actualpos);
 
   // Sync up overdubbing of the loop to a newly created pulse
   void SyncUp();
@@ -571,17 +667,17 @@ public:
 
   // Fades input samples out of mix- writing to 'dest'
   void FadeOut_Input(nframes_t len, 
-		     sample_t *input_l, sample_t *input_r,
-		     sample_t *loop_l, sample_t *loop_r, 
-		     float old_fb, float new_fb, float fb_delta,
-		     sample_t *dest_l, sample_t *dest_r);
+                     sample_t *input_l, sample_t *input_r,
+                     sample_t *loop_l, sample_t *loop_r, 
+                     float old_fb, float new_fb, float fb_delta,
+                     sample_t *dest_l, sample_t *dest_r);
 
   // Fades input samples into mix- writing to 'dest'
   void FadeIn_Input(nframes_t len, 
-		    sample_t *input_l, sample_t *input_r,
-		    sample_t *loop_l, sample_t *loop_r, 
-		    float old_fb, float new_fb, float fb_delta,
-		    sample_t *dest_l, sample_t *dest_r);
+                    sample_t *input_l, sample_t *input_r,
+                    sample_t *loop_l, sample_t *loop_r, 
+                    float old_fb, float new_fb, float fb_delta,
+                    sample_t *dest_l, sample_t *dest_r);
 
   // Jumps to a position within an overdubbing loop- fade of input & output
   void Jump(nframes_t ofs);
@@ -621,7 +717,11 @@ public:
 
   int endsyncidx;   // Pulse sync index for delayed end-of-record
   char endsyncwait; // Are we waiting for a delayed end-of-record?
-
+  
+  int sync_idx;           // Index of sync callback added (or -1 if none is being used)
+  nframes_t sync_add_pos; // Position in pulse where to add sync callback
+  char sync_add;    // RT thread should call AddPulseSync to let pulse know we are waiting for a sync callback
+  
   char stopped,
     growchain, // Nonzero if we should grow the chain of blocks--
                // Zero if record is fixed length
@@ -636,7 +736,7 @@ public:
   float od_playvol, 
     *od_feedback,
     od_feedback_lastval;  // Last value for feedback- used to determine delta, to remove
-			  // zipper noise
+                          // zipper noise
   long od_curbeat; // Current beat in od_loop
   char od_fadein, od_fadeout, od_stop, // Fade in overdub, fade out overdub,
                                        // and stop overdub flags
@@ -647,17 +747,19 @@ public:
     *od_last_lpbuf[2];       // Copy of last loop buffers
 };
 
-class PlayProcessor : public Processor, public EventListener {
+class PlayProcessor : public Processor, public PulseSyncCallback {
 public:
   PlayProcessor(Fweelin *app, Loop *playloop, float playvol, 
-		nframes_t startofs = 0);
+                nframes_t startofs = 0);
   virtual ~PlayProcessor();
 
   // Sync up playing of the loop to a newly created pulse
   void SyncUp();
 
   virtual void process(char pre, nframes_t len, AudioBuffers *ab);
-  
+
+  virtual void PulseSync (int syncidx, nframes_t actualpos);
+
   nframes_t GetPlayedLength();
 
   void SetPlayVol(float newvol) {
@@ -672,16 +774,17 @@ public:
   }
 
   // In Halt() method we ensure that no stray Pulse_Syncs will be responded to
-  virtual void Halt() { sync_state = SS_ENDED; };
-
-  virtual void ReceiveEvent(Event *ev, EventProducer *from);
-  // virtual int ProcessorCall(void *trigger, int data);
+  virtual void Halt() { stopped = 1; sync_state = SS_ENDED; };
 
   SyncStateType sync_state; // Are we waiting for a downbeat, running, ended?
 
   // Pulse to syncronize (quantize) play to
   Pulse *sync; 
 
+  int sync_idx;           // Index of sync callback added (or -1 if none is being used)
+  nframes_t sync_add_pos; // Position in pulse where to add sync callback
+  char sync_add;    // RT thread should call AddPulseSync to let pulse know we are waiting for a sync callback
+
   // Playing in stereo?
   char stereo;
 
@@ -700,27 +803,16 @@ class FileStreamer : public Processor, public EventListener {
   const static nframes_t OUTPUTBUFLEN = 100000;
   const static int MARKERBUFLEN = 50;
 
-  FileStreamer(Fweelin *app, nframes_t outbuflen = OUTPUTBUFLEN);
+  // Initialize a disk stream recording input #input_idx, with the given buffer size
+  FileStreamer(Fweelin *app, int input_idx, char stereo, nframes_t outbuflen = OUTPUTBUFLEN);
   virtual ~FileStreamer();
 
   virtual void process(char pre, nframes_t len, AudioBuffers *ab);
   virtual void ReceiveEvent(Event *ev, EventProducer *from);
-  // virtual int ProcessorCall(void *trigger, int data);
 
   // Starts writing to a new audio stream
   // Note all the heavy work is done in the encode thread!
-  int StartWriting(char *filename, char *timingname, codec type) {
-    if (outname != 0) 
-      return -1; // Already writing!
-
-    outname = filename;
-    outputsize = 0;
-    filetype = type;
-    this->timingname = timingname;
-    writerstatus = STATUS_START_PENDING;
-
-    return 0;
-  };
+  int StartWriting(const std::string &filename_stub, const char *stream_type_name, char write_timing, codec type);
 
   // Stop writing to a stream
   // Note all the heavy work is done in the encode thread!
@@ -729,6 +821,7 @@ class FileStreamer : public Processor, public EventListener {
   };
 
   char GetStatus() { return writerstatus; };
+  const std::string &GetOutputName() { return outname; };
 
   // Returns the number of *frames* written so far-- not bytes
   // The actual file size will vary depending on the codec used
@@ -736,6 +829,7 @@ class FileStreamer : public Processor, public EventListener {
 
   static void *run_encode_thread (void *ptr);
 
+  // Writer status flags
   const static char STATUS_STOPPED = 0,
     STATUS_RUNNING = 1,
     STATUS_STOP_PENDING = 2,
@@ -745,21 +839,21 @@ class FileStreamer : public Processor, public EventListener {
   void InitStreamer();
   void EndStreamer();
   
-  char writerstatus;  
-
-  // Encoding in stereo?
-  char stereo;
-  codec filetype;
+  char writerstatus;     // Status flag
+  int input_idx;         // Index of input to record
+  char stereo;           // Encoding in stereo?
+  codec filetype;        // Audio file type
 
   // File
-  FILE *outfd,   // Current output filedescriptor (audio)
-    *timingfd;   // Current timing output filedescriptor (data- USX)
-  char *outname, // Current output filename
-    *timingname; // Current timing output filename
+  FILE *outfd,         // Current output filedescriptor (audio)
+    *timingfd;         // Current timing output filedescriptor (data- USX)
+  std::string outname, // Current output filename
+    timingname;        // Current timing output filename
+  char write_timing;   // Nonzero if a timing file is being written by this streamer
 
-  // Time markers for storing downbeat points along with audio
+  // Time markers for storing downbeat points along with audio (ring buffer)
   TimeMarker *marks;
-  int mkwriteidx,
+  volatile int mkwriteidx,
     mkreadidx;
 
   // Number of beat triggers passed in this recording 
@@ -768,10 +862,10 @@ class FileStreamer : public Processor, public EventListener {
   nframes_t startcnt;
 
   // Output buffers
-  sample_t *outbuf[2];
-  nframes_t outbuflen,
-    outpos, // Current position of realtime data coming into output buffer
-    encodepos; // Current position of encoder in buffer (lags behind outpos)
+  sample_t *outbuf[2];       // Two channel ring buffer
+  nframes_t outbuflen;
+  volatile nframes_t outpos, // Current position of realtime data coming into output buffer
+    encodepos;               // Current position of encoder in buffer (lags behind outpos)
   char wrap; // Nonzero if outpos has wrapped but encodepos hasn't yet
 
   // Number of bytes written to output file
@@ -785,7 +879,7 @@ class FileStreamer : public Processor, public EventListener {
   iFileEncoder *enc;
 };
 
-// PassthroughProcessor creates a monitor mix of all inputs into the given output
+// PassthroughProcessor creates a monitor mix of several given inputs into one output
 class PassthroughProcessor : public Processor {
 public:
   PassthroughProcessor(Fweelin *app, InputSettings *iset, float *inputvol);
