@@ -1,21 +1,55 @@
 #ifndef __FWEELIN_MIDIIO_H
 #define __FWEELIN_MIDIIO_H
 
+/* Copyright 2004-2011 Jan Pekau
+   
+   This file is part of Freewheeling.
+   
+   Freewheeling is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
+   
+   Freewheeling is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
+   You should have received a copy of the GNU General Public License
+   along with Freewheeling.  If not, see <http://www.gnu.org/licenses/>. */
+
 #ifdef __MACOSX__
 #include <CoreMIDI/MIDIServices.h>
 // Small MIDI output buffer- must be large enough to hold any one MIDI message
-#define OBUF_LEN 128	
+#define OBUF_LEN 128    
 #else
 #include <alsa/asoundlib.h>
 #endif
 
 #include "fweelin_event.h"
 
-#define MIDI_CLOCK_FREQUENCY 24	  // 24 MIDI clock messages per quarter note (beat) MIDI standard
+#define MIDI_CLOCK_FREQUENCY 24   // 24 MIDI clock messages per quarter note (beat) MIDI standard
 
 class Fweelin;
 class PatchItem;
 
+class BypassInfo {
+public:
+  BypassInfo () : notepresscnt(0), releasecnt(0), bypasslen1(0), bypasslen2(0), numheld(0), sp(0), active(0), bypassed(1),
+    bypasscc(0) {};
+  
+  nframes_t notepresscnt,  // Sample count when this channel last had a note pressed
+    releasecnt,            // Sample count when all notes were last released on this channel
+    bypasslen1,            // Number of audio samples (time) of channel inactivity with notes sustained before this channel is bypassed
+    bypasslen2;            // Number of audio samples (time) of channel inactivity with notes released before this channel is bypassed
+    
+  int numheld;          // Number of notes held down on this port/channel
+  char sp,              // Sustain pedal is down for this port/channel
+    active,             // Auto-Bypassing is enabled for this port/channel?
+    bypassed,           // Is this channel bypassed?
+    bypasscc;           // MIDI CC to send for bypass
+};
+
 class MidiIO : public EventProducer, public EventListener {
   friend class Fweelin;
   
@@ -32,7 +66,7 @@ public:
   int GetBenderTune() { return bendertune; };
   void ResetBenderTune() { bendertune = 0; };
 
-  // Sets MIDI echo to one or more ports and channels based on:
+  // Sets up MIDI (echo and auto-bypass). Echo is set to one or more ports and channels based on:
   // a default MIDI output port (def_port)
   // and the given patch. 
   //
@@ -46,7 +80,7 @@ public:
   // at the same time. 
   
   // Zones can overlap, so that one key sends to multiple channels/ports
-  void SetMIDIEcho (int def_port, PatchItem *patch);
+  void SetMIDIForPatch (int def_port, PatchItem *patch);
 
   // Send bank and program change messages where appropriate for the given
   // patch
@@ -101,11 +135,13 @@ protected:
   // Methods that send one MIDI message to one system MIDI out
   void OutputController (int port, int chan, int ctrl, int val);
   void OutputProgramChange (int port, int chan, int val);
+  void OutputChannelPressure (int port, int chan, int val);
   void OutputPitchBend (int port, int chan, int val);
   void OutputNote (int port, int chan, char down, int notenum, int vel);
 
   // MIDI Sync messages
   void OutputClock (int port);
+  void OutputSPP (int port);
   void OutputStart (int port);
   void OutputStop (int port);
   
@@ -113,9 +149,9 @@ protected:
   void OutputStartOnPort ();       // First message for this port in this pass
   void OutputEndOnPort (int port); // Last message for this port in this pass
 
-  // Echo MIDI event to a single port and channel
+  // Echo MIDI event to a single port and channel, and update bypass settings for the given channel
   // Return the port echoed to
-  int EchoEventToPortChannel (Event *ev, int port, int channel);
+  int EchoEventToPortChannel (Event *ev, int port, int channel, int bypasschannel);
 
   // Echo MIDI event back to MIDI outs, according to current patch settings
   void EchoEvent (Event *ev);
@@ -125,23 +161,29 @@ protected:
   void ReceiveNoteOffEvent (int channel, int notenum, int vel);
   void ReceiveNoteOnEvent (int channel, int notenum, int vel);
   void ReceivePitchBendEvent (int channel, int value);
+  void ReceiveChannelPressureEvent (int channel, int value);
   void ReceiveProgramChangeEvent (int channel, int value);
   void ReceiveControlChangeEvent (int channel, int ctrl, int value);
 
   // Send bank and program change message to port/channel
   void SendBankProgramChangeToPortChannel (int bank, int program, 
-					   int port, int channel);
+                                           int port, int channel);
  
+  void CheckBypass();   // Time to bypass any channels yet?
+  void CheckUnbypass(); // Unbypass channels still bypassed but needing use
+  nframes_t checkfreq,  // How often (samples) to check whether to bypass unused channels
+    lastchecktime;      // Last time (samples) auto-bypass conditions checked
+  
 #ifdef __MACOSX__
   MIDIClientRef client;
   MIDIPortRef *in_ports, *out_ports;
   MIDIEndpointRef *out_sources;
-  MIDIEndpointRef dest;	// Deprecated
-  int inputidx;		// Index of last selected MIDI input source
+  MIDIEndpointRef dest; // Deprecated
+  int inputidx;         // Index of last selected MIDI input source
   
-  MIDIPacket *curPacket;	
+  MIDIPacket *curPacket;        
   MIDIPacketList *packetList;  
-  Byte obuf[OBUF_LEN];	// Small MIDI event output buffer
+  Byte obuf[OBUF_LEN];  // Small MIDI event output buffer
 
   void static MidiInputProc (const MIDIPacketList *pktlist, void *refCon, void *connRefCon);  
 #else
@@ -158,9 +200,16 @@ protected:
   // For each MIDI note on the scale, what default port and patch was the note
   // played with? This allows us to send note off(s) to the right place(s), 
   // even when the patch is changed while notes are held
+  
+  // Note: this assumes only 1 keyboard
   int *note_def_port; 
   PatchItem **note_patch;
   
+  // Data for automatic toggling of Soft-synth Bypass based on active MIDI channels
+  // One instance for each MIDI channel and port
+  BypassInfo *bp;
+  BypassInfo *getBP (int port, int channel);
+  
   // MIDI Sync
   int midisyncxmit;  // Nonzero if we should transmit MIDI sync messages
 };
